import 'dart:typed_data';

import 'package:base_utils/base/lifecycle.dart';
import 'package:base_utils/router/router.dart';
import 'package:base_utils/utils/string_utils.dart';
import 'package:flutter/material.dart';
import 'package:flutter_advanced_networkimage/flutter_advanced_networkimage.dart';
//import 'package:flutter_advanced_networkimage/provider.dart';

/// Provides common utilities and functions to build ui and handle app lifecycle
abstract class BaseState<T extends StatefulWidget> extends LifeCycleState<T> {
  Size screenSize;

  Size designScreenSize;

  /// Called when the app is temporarily closed or a new route is pushed
  @override
  void onPause() {}

  /// Called when users return to the app or the adjacent route of this widget is popped
  @override
  void onResume() {}

  /// Called onnce when this state's widget finished building
  @override
  void onFirstFrame() {}

  @override
  void didChangeDependencies() {
    super.didChangeDependencies();
    screenSize = MediaQuery.of(context).size;
    designScreenSize = getDesignSize() ?? Size(375.0, 812.0);
  }

  /// Override to define new design size for ui
  Size getDesignSize() => null;

  String get packageName => null;

  /// Scale the provided 'designSize' proportionally to the screen's width
  double scaleWidth(num designSize, {bool preventScaleDown = false}) {
    if (designSize == null) return null;
    final scaledSize = screenSize.width * designSize / designScreenSize.width;

    if (preventScaleDown && scaledSize < designSize) {
      return designSize?.toDouble();
    }

    return scaledSize;
  }

  /// Scale the provided 'designSize' proportionally to the screen's height
  double scaleHeight(num designSize, {bool preventScaleDown = false}) {
    if (designSize == null) return null;
    final scaledSize = screenSize.height * designSize / designScreenSize.height;

    if (preventScaleDown && scaledSize < designSize) {
      return designSize?.toDouble();
    }

    return scaledSize;
  }

  Widget buildAssetsImage(String path,
      {BoxFit fit = BoxFit.contain, num width, num height, String package}) {
    return Image.asset(path,
        height: height?.toDouble(),
        width: width?.toDouble(),
        package: package ?? packageName,
        fit: fit);
  }

  DecorationImage buildDecorationImage(String path,
      {BoxFit fit = BoxFit.contain, ColorFilter filter, String package}) {
    return DecorationImage(
        image: AssetImage(path, package: package ?? packageName),
        fit: fit,
        colorFilter: filter);
  }

  Widget buildRemoteImage(
    String url, {
    num height,
    num width,
    BoxFit fit = BoxFit.contain,
    double scale = 1.0,
    Map<String, String> header,
    bool useDiskCache = false,
    Function loadedCallback,
    Function loadFailedCallback,
    Uint8List fallbackImage,
        BorderRadius borderRadius = BorderRadius.zero,
  }) {
    if (isEmpty(url)) return Container();

    return ClipRRect(
      borderRadius: borderRadius,
      child: Image(
        height: height?.toDouble(),
        width: width?.toDouble(),
        fit: fit,
        image: AdvancedNetworkImage(
          url,
          scale: scale,
          fallbackImage: fallbackImage,
          header: header,
          loadedCallback: loadedCallback,
          loadFailedCallback: loadFailedCallback,
          useDiskCache: useDiskCache,
          timeoutDuration: Duration(seconds: 30),
          retryLimit: 10,
          retryDuration: Duration(milliseconds: 2800),
        ),
      ),
    );
  }

  @override
  RouteObserver<Route> get routeObserver => Router.instance.routeObserver;
}
